import paho.mqtt.client as mqtt
import threading
from loguru import logger


def mqttOnConnectCallback(client, userdata, flags, rc):
    userdata.onConnect()

def mqttOnMessageCallback(client, userdata, message):
    userdata.onMessage(message.topic, message.payload)

def mqttOnDisconnectCallback(client, userdata, rc):
    userdata.onDisconnect(rc)


class AbstractMqttHandler(threading.Thread):
    def __init__(self, config):
        super().__init__()

        self.config = config

        self.client = mqtt.Client(userdata=self)

        # consider this flag in the localLoop
        self.poisonPill = False

    def run(self):
        self.client.on_message = mqttOnMessageCallback
        self.client.on_connect = mqttOnConnectCallback
        self.client.on_disconnect = mqttOnDisconnectCallback

        if ("login" in self.config["mqtt"]) and ("password" in self.config["mqtt"]):
            self.client.username_pw_set(self.config["mqtt"]["login"], self.config["mqtt"]["password"])
        
        self.client.connect(self.config["mqtt"]["broker"])
        self.client.loop_start()
        logger.debug("mqtt loop started")

        self.localLoop()
    
    def localLoop(self):
        raise NotImplementedError()

    def stop(self):
        self.client.loop_stop()
        self.client.disconnect()
        logger.debug("about to stop loop")
        
        self.poisonPill = True
        logger.debug("kill flag set")

    def onConnect(self):
        logger.info("mqtt connected")
    
    def onDisconnect(self, rc):
        logger.warning("disconnect, rc: {}".format(rc))

    def onMessage(self, topic, payload):
        logger.warning("unexpected message received: {} -> {}".format(topic, str(payload)))

