import threading
from loguru import logger
from DataObject import DataObject


POISON_PILL = DataObject(name="PoisonPill", topic="kill", payload=None)


class AbstractSinkHandler(threading.Thread):
    def __init__(self, config, name, inQueue, experiment):
        super().__init__()

        self.config = config
        self.localConfig = self.config[name]
        self.name = name
        self.inQueue = inQueue
        self.experiment = experiment

    def run(self):
        logger.debug("loop started")

        self.init()

        while True:
            dataObject = self.inQueue.get()
            if (dataObject == POISON_PILL):
                logger.debug("swallowed the poison pill")
                break
            self.sinkAction(dataObject)            

    def init(self):
        logger.debug("no initialization needed")

    def sinkAction(self, dataObject):
        raise NotImplementedError()

    def stop(self):
        self.inQueue.put(POISON_PILL)
        logger.debug("kill flag set")
