from loguru import logger
import os
import configparser
import json
import argparse
import importlib
import sys
import paho.mqtt.client as mqtt
from time import sleep

DEFAULT_CONFIG_FILE = './config/dummyClientConfig.ini'
    



parser = argparse.ArgumentParser(description="DummyClient.py")
parser.add_argument('--operation', '-o',
                    help='Operation to perform.',
                    required=True)
parser.add_argument('--params', '-p',
                    help='JSON string with parameter for the selected operation, default: {}',                    
                    required=False,
                    default="{}")
parser.add_argument('--config', '-c',
                    help='configuration file, default: {}'.format(DEFAULT_CONFIG_FILE),                    
                    required=False,
                    default=DEFAULT_CONFIG_FILE)
parser.add_argument('--verbosity', '-v',
                    help='Minimal log level for output: DEBUG, INFO, WARNING, ..., default: DEBUG',
                    required=False,
                    default="DEBUG")                    
parser.add_argument('--nocolorize', '-n',
                    help='disable colored output (for cron)',
                    required=False,
                    action='store_true',
                    default=False)

args = parser.parse_args()
operation = args.operation
params = json.loads(args.params)
logLevel = args.verbosity
noColorize = args.nocolorize

config = configparser.ConfigParser()
config.read(args.config)


logger.remove()
logger.add(sys.stderr, colorize=(not noColorize), level=logLevel)

logger.info("Start DummyClient")
try:
    client = mqtt.Client()
    if ("login" in config["mqtt"]) and ("password" in config["mqtt"]):
        client.username_pw_set(config["mqtt"]["login"], config["mqtt"]["password"])
    client.connect(config["mqtt"]["broker"])
    client.loop_start()
    logger.debug("mqtt loop started")

    opMod = importlib.import_module(operation)
    logger.info(f"Operation {operation} loaded, about to perform operation")
    opMod.perform(client, params)
    logger.info("Operation returned")
except Exception as err:
    logger.error("Exception: {}".format(str(err)))
    raise err
finally:
    logger.info("About to shutdown mqtt connection")
    client.disconnect()
    client.loop_stop()
    logger.info("mqtt connection down")


