from json.decoder import JSONDecodeError
import paho.mqtt.client as mqtt
from loguru import logger
from time import sleep
import json

from AbstractMqttHandler import AbstractMqttHandler
from DataObject import DataObject

class GenericMqttSubscriber(AbstractMqttHandler):
    def __init__(self, config, configId, queue):
        super().__init__(config)
        localConfig = self.config[configId]
        self.dataObjectName = localConfig["dataObjectName"]
        self.subscribeTopic = localConfig["topic"]
        self.queue = queue

    def localLoop(self):
        while not self.poisonPill:
            sleep(float(1.0))

    def onMessage(self, topic, payload):
        try: 
            logger.info("message received: {} -> {}".format(topic, str(payload)))
            dataObject = DataObject(name=self.dataObjectName, topic=topic, payload=json.loads(payload))
            self.queue.put(dataObject)
        except JSONDecodeError as e:
            logger.warning("unable to decode {} to json".format(str(payload)))

    def onConnect(self):
        logger.debug("connected")
        self.client.subscribe(self.subscribeTopic)
        logger.debug(f"subscribed to {self.subscribeTopic}")
