from loguru import logger
import argparse
import configparser
import threading
from queue import Queue
import signal

from GenericMqttSubscriber import GenericMqttSubscriber
from RigCmdHandler import RigCmdHandler


knell = threading.Event()

def exceptHook(args):
    global deathBell
    logger.error("Exception in thread caught: {}".format(args))
    knell.set()
    logger.error("rang the knell")

def sigintHook(sig, frame):
    logger.error("SIGINT received")
    knell.set()
    logger.error("rang the knell")

# ------------------------------------------------------------------------------
logger.info("IIoTFeeder2 starting")

parser = argparse.ArgumentParser(description="IIoTFeeder2")
parser.add_argument('--config', '-f',
                    help='Config file, default is $pwd/config/iiotFeederConfig.ini',
                    required=False,
                    default='./config/iiotFeederConfig.ini')
args = parser.parse_args()

config = configparser.ConfigParser()
config.read(args.config)

signal.signal(signal.SIGINT, sigintHook)
logger.debug("SIGINT hook set")

# ------------------------------------------------------------------------------
masterQueue = Queue()
masterSubscriber = GenericMqttSubscriber(config, "master", masterQueue)
masterSubscriber.start()
logger.debug("MasterSubscriber started")

rigCmdHandler = RigCmdHandler(config, masterQueue)
rigCmdHandler.start()
logger.debug("RigCmdHandler started")

# ------------------------------------------------------------------------------
threading.excepthook = exceptHook
logger.debug("Threading excepthook set")

logger.debug("IIoTFeeder2 running")

# ------------------------------------------------------------------------------
while not knell.wait(1.0):
    # logger.debug("master queue size {}".format(masterQueue.qsize()))
    pass

logger.error("IIoTFeeder2 is dying")

masterSubscriber.stop()
masterSubscriber.join()
logger.error("masterSubscriber joined")

rigCmdHandler.stop()
rigCmdHandler.join()
logger.error("rigCmdHandler joined")

# ------------------------------------------------------------------------------
logger.info("IIoTFeeder2 to terminate")


